@@ -1,5 +1,5 @@
 /* =========================================================================
-   Copyright (c) 2010-2013, Institute for Microelectronics,
+   Copyright (c) 2010-2014, Institute for Microelectronics,
                             Institute for Analysis and Scientific Computing,
                             TU Wien.
    Portions of this software are copyright by UChicago Argonne, LLC.
@@ -16,6 +16,11 @@
 ============================================================================= */
 
 
+
+/** \file tests/src/scheduler_matrix_vector.cpp  Tests the scheduler for matrix-vector-operations.
+*   \test Tests the scheduler for matrix-vector-operations.
+**/
+
 //
 // *** System
 //
@@ -56,7 +61,7 @@ using namespace boost::numeric;
 //
 // -------------------------------------------------------------
 //
-template <typename ScalarType>
+template<typename ScalarType>
 ScalarType diff(ScalarType & s1, viennacl::scalar<ScalarType> & s2)
 {
    viennacl::backend::finish();
@@ -65,7 +70,7 @@ ScalarType diff(ScalarType & s1, viennacl::scalar<ScalarType> & s2)
    return 0;
 }
 
-template <typename ScalarType, typename VCLVectorType>
+template<typename ScalarType, typename VCLVectorType>
 ScalarType diff(ublas::vector<ScalarType> const & v1, VCLVectorType const & v2)
 {
    ublas::vector<ScalarType> v2_cpu(v2.size());
@@ -83,7 +88,7 @@ ScalarType diff(ublas::vector<ScalarType> const & v1, VCLVectorType const & v2)
    return norm_inf(v2_cpu);
 }
 
-template <typename ScalarType, typename VCLMatrixType>
+template<typename ScalarType, typename VCLMatrixType>
 ScalarType diff(ublas::matrix<ScalarType> const & mat1, VCLMatrixType const & mat2)
 {
    ublas::matrix<ScalarType> mat2_cpu(mat2.size1(), mat2.size2());
@@ -108,7 +113,7 @@ ScalarType diff(ublas::matrix<ScalarType> const & mat1, VCLMatrixType const & ma
 // -------------------------------------------------------------
 //
 
-template <typename NumericT, typename Epsilon,
+template<typename NumericT, typename Epsilon,
           typename UblasMatrixType, typename UblasVectorType,
           typename VCLMatrixType, typename VCLVectorType1, typename VCLVectorType2>
 int test_prod_rank1(Epsilon const & epsilon,
@@ -135,7 +140,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -149,7 +154,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -163,7 +168,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -179,7 +184,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -195,7 +200,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -211,7 +216,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -232,7 +237,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product with scaled additions" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -249,7 +254,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product with scaled additions" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -266,7 +271,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
+   if ( std::fabs(diff(ublas_v1, vcl_v1)) > epsilon )
    {
       std::cout << "# Error at operation: matrix-vector product with scaled additions" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v1, vcl_v1)) << std::endl;
@@ -285,7 +290,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
+   if ( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
    {
       std::cout << "# Error at operation: transposed matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v2, vcl_v2)) << std::endl;
@@ -299,7 +304,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
+   if ( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
    {
       std::cout << "# Error at operation: transposed matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v2, vcl_v2)) << std::endl;
@@ -313,7 +318,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
+   if ( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
    {
       std::cout << "# Error at operation: transposed matrix-vector product" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v2, vcl_v2)) << std::endl;
@@ -328,7 +333,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
+   if ( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
    {
       std::cout << "# Error at operation: transposed matrix-vector product with scaled additions" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v2, vcl_v2)) << std::endl;
@@ -342,7 +347,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
+   if ( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
    {
       std::cout << "# Error at operation: transposed matrix-vector product with scaled additions" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v2, vcl_v2)) << std::endl;
@@ -356,7 +361,7 @@ int test_prod_rank1(Epsilon const & epsilon,
    viennacl::scheduler::execute(my_statement);
    }
 
-   if( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
+   if ( std::fabs(diff(ublas_v2, vcl_v2)) > epsilon )
    {
       std::cout << "# Error at operation: transposed matrix-vector product with scaled additions" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(ublas_v2, vcl_v2)) << std::endl;
@@ -442,7 +447,7 @@ int test(Epsilon const& epsilon)
    }
    vcl_matrix.resize(2*num_rows, 2*num_cols, true);
    viennacl::copy(vcl_matrix, matrix);
-   if( std::fabs(diff(matrix, vcl_matrix)) > epsilon )
+   if ( std::fabs(diff(matrix, vcl_matrix)) > epsilon )
    {
       std::cout << "# Error at operation: matrix resize (to larger)" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(matrix, vcl_matrix)) << std::endl;
@@ -459,7 +464,7 @@ int test(Epsilon const& epsilon)
    std::cout << "Matrix resizing (to smaller)" << std::endl;
    matrix.resize(result.size(), rhs.size(), true);
    vcl_matrix.resize(result.size(), rhs.size(), true);
-   if( std::fabs(diff(matrix, vcl_matrix)) > epsilon )
+   if ( std::fabs(diff(matrix, vcl_matrix)) > epsilon )
    {
       std::cout << "# Error at operation: matrix resize (to smaller)" << std::endl;
       std::cout << "  diff: " << std::fabs(diff(matrix, vcl_matrix)) << std::endl;
@@ -872,7 +877,7 @@ int main()
       std::cout << "  numeric: float" << std::endl;
       std::cout << "  layout: row-major" << std::endl;
       retval = test<NumericT, viennacl::row_major>(epsilon);
-      if( retval == EXIT_SUCCESS )
+      if ( retval == EXIT_SUCCESS )
          std::cout << "# Test passed" << std::endl;
       else
          return retval;
@@ -888,7 +893,7 @@ int main()
       std::cout << "  numeric: float" << std::endl;
       std::cout << "  layout: column-major" << std::endl;
       retval = test<NumericT, viennacl::column_major>(epsilon);
-      if( retval == EXIT_SUCCESS )
+      if ( retval == EXIT_SUCCESS )
          std::cout << "# Test passed" << std::endl;
       else
          return retval;
@@ -899,7 +904,7 @@ int main()
 
 
 #ifdef VIENNACL_WITH_OPENCL
-   if( viennacl::ocl::current_device().double_support() )
+   if ( viennacl::ocl::current_device().double_support() )
 #endif
    {
       {
@@ -910,7 +915,7 @@ int main()
          std::cout << "  numeric: double" << std::endl;
          std::cout << "  layout: row-major" << std::endl;
          retval = test<NumericT, viennacl::row_major>(epsilon);
-            if( retval == EXIT_SUCCESS )
+            if ( retval == EXIT_SUCCESS )
                std::cout << "# Test passed" << std::endl;
             else
               return retval;
@@ -926,7 +931,7 @@ int main()
          std::cout << "  numeric: double" << std::endl;
          std::cout << "  layout: column-major" << std::endl;
          retval = test<NumericT, viennacl::column_major>(epsilon);
-            if( retval == EXIT_SUCCESS )
+            if ( retval == EXIT_SUCCESS )
                std::cout << "# Test passed" << std::endl;
             else
               return retval;
