@@ -1,5 +1,5 @@
 /* =========================================================================
-   Copyright (c) 2010-2013, Institute for Microelectronics,
+   Copyright (c) 2010-2014, Institute for Microelectronics,
                             Institute for Analysis and Scientific Computing,
                             TU Wien.
    Portions of this software are copyright by UChicago Argonne, LLC.
@@ -15,6 +15,11 @@
    License:         MIT (X11), see file LICENSE in the base directory
 ============================================================================= */
 
+
+/** \file tests/src/scheduler_sparse.cpp  Tests the scheduler for sparse matrix operations.
+*   \test Tests the scheduler for sparse matrix operations.
+**/
+
 #ifndef NDEBUG
  #define NDEBUG
 #endif
@@ -66,7 +71,7 @@ using namespace boost::numeric;
 //
 // -------------------------------------------------------------
 //
-template <typename ScalarType>
+template<typename ScalarType>
 ScalarType diff(ScalarType & s1, viennacl::scalar<ScalarType> & s2)
 {
    if (s1 != s2)
@@ -74,7 +79,7 @@ ScalarType diff(ScalarType & s1, viennacl::scalar<ScalarType> & s2)
    return 0;
 }
 
-template <typename ScalarType>
+template<typename ScalarType>
 ScalarType diff(ublas::vector<ScalarType> & v1, viennacl::vector<ScalarType> & v2)
 {
    ublas::vector<ScalarType> v2_cpu(v2.size());
@@ -106,7 +111,7 @@ ScalarType diff(ublas::vector<ScalarType> & v1, viennacl::vector<ScalarType> & v
 }
 
 
-template <typename ScalarType, typename VCL_MATRIX>
+template<typename ScalarType, typename VCL_MATRIX>
 ScalarType diff(ublas::compressed_matrix<ScalarType> & cpu_matrix, VCL_MATRIX & gpu_matrix)
 {
   typedef ublas::compressed_matrix<ScalarType>  CPU_MATRIX;
@@ -184,7 +189,7 @@ int test(Epsilon const& epsilon)
   ublas::vector<NumericT> result;
   ublas::compressed_matrix<NumericT> ublas_matrix;
 
-  if (viennacl::io::read_matrix_market_file(ublas_matrix, "../../examples/testdata/mat65k.mtx") == EXIT_FAILURE)
+  if (viennacl::io::read_matrix_market_file(ublas_matrix, "../examples/testdata/mat65k.mtx") == EXIT_FAILURE)
   {
     std::cout << "Error reading Matrix file" << std::endl;
     return EXIT_FAILURE;
@@ -224,7 +229,7 @@ int test(Epsilon const& epsilon)
   }
   vcl_result = viennacl::linalg::prod(vcl_compressed_matrix, vcl_rhs);
 
-  if( std::fabs(diff(result, vcl_result)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product with compressed_matrix" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result)) << std::endl;
@@ -240,7 +245,7 @@ int test(Epsilon const& epsilon)
   viennacl::scheduler::execute(my_statement);
   }
 
-  if( std::fabs(diff(result, vcl_result)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product with coordinate_matrix" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result)) << std::endl;
@@ -253,7 +258,7 @@ int test(Epsilon const& epsilon)
   viennacl::scheduler::execute(my_statement);
   }
 
-  if( std::fabs(diff(result, vcl_result2)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result2)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product (coordinate_matrix) with scaled additions" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result2)) << std::endl;
@@ -276,7 +281,7 @@ int test(Epsilon const& epsilon)
   }
   vcl_result = viennacl::linalg::prod(vcl_ell_matrix, vcl_rhs);
 
-  if( std::fabs(diff(result, vcl_result)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product with ell_matrix" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result)) << std::endl;
@@ -298,7 +303,7 @@ int test(Epsilon const& epsilon)
   viennacl::scheduler::execute(my_statement);
   }
 
-  if( std::fabs(diff(result, vcl_result)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product with hyb_matrix" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result)) << std::endl;
@@ -325,7 +330,7 @@ int test(Epsilon const& epsilon)
   viennacl::scheduler::execute(my_statement);
   }
 
-  if( std::fabs(diff(result, vcl_result2)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result2)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product (compressed_matrix) with scaled additions" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result2)) << std::endl;
@@ -343,7 +348,7 @@ int test(Epsilon const& epsilon)
   viennacl::scheduler::execute(my_statement);
   }
 
-  if( std::fabs(diff(result, vcl_result2)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result2)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product (coordinate_matrix) with scaled additions" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result2)) << std::endl;
@@ -360,7 +365,7 @@ int test(Epsilon const& epsilon)
   viennacl::scheduler::execute(my_statement);
   }
 
-  if( std::fabs(diff(result, vcl_result2)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result2)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product (ell_matrix) with scaled additions" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result2)) << std::endl;
@@ -377,7 +382,7 @@ int test(Epsilon const& epsilon)
   viennacl::scheduler::execute(my_statement);
   }
 
-  if( std::fabs(diff(result, vcl_result2)) > epsilon )
+  if ( std::fabs(diff(result, vcl_result2)) > epsilon )
   {
     std::cout << "# Error at operation: matrix-vector product (hyb_matrix) with scaled additions" << std::endl;
     std::cout << "  diff: " << std::fabs(diff(result, vcl_result2)) << std::endl;
@@ -413,7 +418,7 @@ int main()
     std::cout << "  eps:     " << epsilon << std::endl;
     std::cout << "  numeric: float" << std::endl;
     retval = test<NumericT>(epsilon);
-    if( retval == EXIT_SUCCESS )
+    if ( retval == EXIT_SUCCESS )
         std::cout << "# Test passed" << std::endl;
     else
         return retval;
@@ -423,7 +428,7 @@ int main()
   std::cout << std::endl;
 
 #ifdef VIENNACL_WITH_OPENCL
-  if( viennacl::ocl::current_device().double_support() )
+  if ( viennacl::ocl::current_device().double_support() )
 #endif
   {
     {
@@ -433,7 +438,7 @@ int main()
       std::cout << "  eps:     " << epsilon << std::endl;
       std::cout << "  numeric: double" << std::endl;
       retval = test<NumericT>(epsilon);
-      if( retval == EXIT_SUCCESS )
+      if ( retval == EXIT_SUCCESS )
         std::cout << "# Test passed" << std::endl;
       else
         return retval;
