@@ -1,5 +1,5 @@
 /* =========================================================================
-   Copyright (c) 2010-2013, Institute for Microelectronics,
+   Copyright (c) 2010-2014, Institute for Microelectronics,
                             Institute for Analysis and Scientific Computing,
                             TU Wien.
    Portions of this software are copyright by UChicago Argonne, LLC.
@@ -15,6 +15,12 @@
    License:         MIT (X11), see file LICENSE in the base directory
 ============================================================================= */
 
+
+
+/** \file tests/src/scheduler_vector.cpp  Tests the scheduler for vector-operations.
+*   \test Tests the scheduler for vector-operations.
+**/
+
 //
 // *** System
 //
@@ -52,40 +58,40 @@ using namespace boost::numeric;
 //
 // -------------------------------------------------------------
 //
-template <typename ScalarType>
+template<typename ScalarType>
 ScalarType diff(ScalarType const & s1, ScalarType const & s2)
 {
    viennacl::backend::finish();
-   if (s1 != s2)
+   if (std::fabs(s1 - s2) > 0)
       return (s1 - s2) / std::max(std::fabs(s1), std::fabs(s2));
    return 0;
 }
 //
 // -------------------------------------------------------------
 //
-template <typename ScalarType>
+template<typename ScalarType>
 ScalarType diff(ScalarType const & s1, viennacl::scalar<ScalarType> const & s2)
 {
    viennacl::backend::finish();
-   if (s1 != s2)
+   if (std::fabs(s1 - s2) > 0)
       return (s1 - s2) / std::max(std::fabs(s1), std::fabs(s2));
    return 0;
 }
 //
 // -------------------------------------------------------------
 //
-template <typename ScalarType>
+template<typename ScalarType>
 ScalarType diff(ScalarType const & s1, viennacl::entry_proxy<ScalarType> const & s2)
 {
    viennacl::backend::finish();
-   if (s1 != s2)
+   if (std::fabs(s1 - s2) > 0)
       return (s1 - s2) / std::max(std::fabs(s1), std::fabs(s2));
    return 0;
 }
 //
 // -------------------------------------------------------------
 //
-template <typename ScalarType, typename ViennaCLVectorType>
+template<typename ScalarType, typename ViennaCLVectorType>
 ScalarType diff(ublas::vector<ScalarType> const & v1, ViennaCLVectorType const & vcl_vec)
 {
    ublas::vector<ScalarType> v2_cpu(vcl_vec.size());
@@ -104,7 +110,7 @@ ScalarType diff(ublas::vector<ScalarType> const & v1, ViennaCLVectorType const &
 }
 
 
-template <typename T1, typename T2>
+template<typename T1, typename T2>
 int check(T1 const & t1, T2 const & t2, double epsilon)
 {
   int retval = EXIT_SUCCESS;
@@ -415,6 +421,83 @@ int test(Epsilon const& epsilon,
     return EXIT_FAILURE;
   }
 
+
+  std::cout << "x = element_pow(x, y)... ";
+  {
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+  {
+    ublas_v1[i] = NumericT(2.0) + random<NumericT>();
+    ublas_v2[i] = NumericT(1.0) + random<NumericT>();
+  }
+  viennacl::copy(ublas_v1, vcl_v1);
+  viennacl::copy(ublas_v2, vcl_v2);
+
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+    ublas_v1[i] = std::pow(ublas_v1[i], ublas_v2[i]);
+  viennacl::scheduler::statement   my_statement(vcl_v1, viennacl::op_assign(), viennacl::linalg::element_pow(vcl_v1, vcl_v2));
+  viennacl::scheduler::execute(my_statement);
+
+  if (check(ublas_v1, vcl_v1, epsilon) != EXIT_SUCCESS)
+    return EXIT_FAILURE;
+  }
+
+  std::cout << "x = element_pow(x + y, y)... ";
+  {
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+  {
+    ublas_v1[i] = NumericT(2.0) + random<NumericT>();
+    ublas_v2[i] = NumericT(1.0) + random<NumericT>();
+  }
+  viennacl::copy(ublas_v1, vcl_v1);
+  viennacl::copy(ublas_v2, vcl_v2);
+
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+    ublas_v1[i] = std::pow(ublas_v1[i]  + ublas_v2[i], ublas_v2[i]);
+  viennacl::scheduler::statement   my_statement(vcl_v1, viennacl::op_assign(), viennacl::linalg::element_pow(vcl_v1 + vcl_v2, vcl_v2));
+  viennacl::scheduler::execute(my_statement);
+
+  if (check(ublas_v1, vcl_v1, epsilon) != EXIT_SUCCESS)
+    return EXIT_FAILURE;
+  }
+
+  std::cout << "x = element_pow(x, x + y)... ";
+  {
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+  {
+    ublas_v1[i] = NumericT(2.0) + random<NumericT>();
+    ublas_v2[i] = NumericT(1.0) + random<NumericT>();
+  }
+  viennacl::copy(ublas_v1, vcl_v1);
+  viennacl::copy(ublas_v2, vcl_v2);
+
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+    ublas_v1[i] = std::pow(ublas_v1[i], ublas_v1[i] + ublas_v2[i]);
+  viennacl::scheduler::statement   my_statement(vcl_v1, viennacl::op_assign(), viennacl::linalg::element_pow(vcl_v1, vcl_v2 + vcl_v1));
+  viennacl::scheduler::execute(my_statement);
+
+  if (check(ublas_v1, vcl_v1, epsilon) != EXIT_SUCCESS)
+    return EXIT_FAILURE;
+  }
+
+  std::cout << "x = element_pow(x - y, y + x)... ";
+  {
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+  {
+    ublas_v1[i] = NumericT(2.0) + random<NumericT>();
+    ublas_v2[i] = NumericT(1.0) + random<NumericT>();
+  }
+  viennacl::copy(ublas_v1, vcl_v1);
+  viennacl::copy(ublas_v2, vcl_v2);
+
+  for (std::size_t i=0; i<ublas_v1.size(); ++i)
+    ublas_v1[i] = std::pow(ublas_v1[i] - ublas_v2[i], ublas_v2[i] + ublas_v1[i]);
+  viennacl::scheduler::statement   my_statement(vcl_v1, viennacl::op_assign(), viennacl::linalg::element_pow(vcl_v1 - vcl_v2, vcl_v2 + vcl_v1));
+  viennacl::scheduler::execute(my_statement);
+
+  if (check(ublas_v1, vcl_v1, epsilon) != EXIT_SUCCESS)
+    return EXIT_FAILURE;
+  }
+
   std::cout << "--- Testing elementwise operations (unary) ---" << std::endl;
 #define GENERATE_UNARY_OP_TEST(OPNAME) \
   ublas_v1 = ublas::scalar_vector<NumericT>(ublas_v1.size(), NumericT(0.21)); \
@@ -423,7 +506,7 @@ int test(Epsilon const& epsilon,
   viennacl::copy(ublas_v2.begin(), ublas_v2.end(), vcl_v2.begin()); \
   { \
   for (std::size_t i=0; i<ublas_v1.size(); ++i) \
-    ublas_v1[i] = OPNAME(ublas_v2[i]); \
+    ublas_v1[i] = std::OPNAME(ublas_v2[i]); \
   viennacl::scheduler::statement my_statement(vcl_v1, viennacl::op_assign(), viennacl::linalg::element_##OPNAME(vcl_v2)); \
   viennacl::scheduler::execute(my_statement); \
   if (check(ublas_v1, vcl_v1, epsilon) != EXIT_SUCCESS) \
@@ -659,7 +742,7 @@ int main()
       std::cout << "  eps:     " << epsilon << std::endl;
       std::cout << "  numeric: float" << std::endl;
       retval = test<NumericT>(epsilon);
-      if( retval == EXIT_SUCCESS )
+      if ( retval == EXIT_SUCCESS )
          std::cout << "# Test passed" << std::endl;
       else
          return retval;
@@ -668,7 +751,7 @@ int main()
    std::cout << "----------------------------------------------" << std::endl;
    std::cout << std::endl;
 #ifdef VIENNACL_WITH_OPENCL
-   if( viennacl::ocl::current_device().double_support() )
+   if ( viennacl::ocl::current_device().double_support() )
 #endif
    {
       {
@@ -678,7 +761,7 @@ int main()
          std::cout << "  eps:     " << epsilon << std::endl;
          std::cout << "  numeric: double" << std::endl;
          retval = test<NumericT>(epsilon);
-         if( retval == EXIT_SUCCESS )
+         if ( retval == EXIT_SUCCESS )
            std::cout << "# Test passed" << std::endl;
          else
            return retval;
